package services

import (
	"bytes"
	"encoding/base64"
	"encoding/binary"
	"encoding/json"
	"errors"
	"eta_gn/eta_task/models"
	"eta_gn/eta_task/services/alarm_msg"
	"eta_gn/eta_task/utils"
	"fmt"
	"github.com/PuerkitoBio/goquery"
	"github.com/kgiannakakis/mp3duration/src/mp3duration"
	"html"
	"io"
	"io/ioutil"
	"os"
	"strings"
	"time"
	"unicode"
)

func CreateVideo(report *models.Report) (err error) {
	defer func() {
		if err != nil {
			utils.FileLog.Error("CreateVideo Err:%s", err.Error())
			go alarm_msg.SendAlarmMsg("CreateVideo, Err:"+err.Error(), 3)
		}
	}()

	conf, e := models.GetBusinessConf()
	if e != nil {
		err = fmt.Errorf("获取基础配置失败, Err: " + e.Error())
		return
	}
	if conf[models.BusinessConfUseXf] != "true" {
		return
	}
	if conf[models.BusinessConfXfAppid] == "" || conf[models.BusinessConfXfApiKey] == "" || conf[models.BusinessConfXfApiSecret] == "" || conf[models.BusinessConfXfVcn] == "" {
		return
	}
	var xfReq XfParams
	xfReq.XfAPPID = conf[models.BusinessConfXfAppid]
	xfReq.XfAPIKey = conf[models.BusinessConfXfApiKey]
	xfReq.XfAPISecret = conf[models.BusinessConfXfApiSecret]

	ct, err := time.Parse(utils.FormatDateTime, report.CreateTime)
	createTime := ct.Format("0102")
	videoName := report.Title + "(" + createTime + ")"
	content := html.UnescapeString(report.Content)
	content = strings.Replace(content, "Powered", "", -1)
	content = strings.Replace(content, "by", "", -1)
	content = strings.Replace(content, "Froala", "", -1)
	content = strings.Replace(content, "Editor", "", -1)
	doc, err := goquery.NewDocumentFromReader(strings.NewReader(content))
	if err != nil {
		return
	}

	param := new(models.XfSendParam)
	param.Common.AppId = conf[models.BusinessConfXfAppid]
	param.Business.Aue = "lame"
	param.Business.Sfl = 1
	param.Business.Auf = "audio/L16;rate=16000"
	param.Business.Vcn = conf[models.BusinessConfXfVcn]
	param.Business.Speed = 50
	param.Business.Volume = 100
	param.Business.Pitch = 50
	param.Business.Bgs = 0
	param.Business.Tte = "UTF8"
	param.Business.Reg = "2"
	param.Business.Rdn = "0"
	param.Data.Status = 2
	videoContent := doc.Text()

	saveName := utils.GetRandStringNoSpecialChar(16) + ".mp3"
	savePath := "./" + saveName

	videoContent = strings.TrimSpace(videoContent)
	if videoContent == `` {
		return
	}

	contentArr := GetChineseCount(videoContent)
	for _, v := range contentArr {
		newText := v
		param.Data.Text = base64.StdEncoding.EncodeToString([]byte(newText))
		result, err := json.Marshal(param)
		if err != nil {
			return err
		}
		err = GetXfVideo(result, savePath, xfReq)
		if err != nil {
			err = errors.New("GetXfVideo Err:" + err.Error())
			utils.FileLog.Error("GetXfVideo err", err.Error())
			return err
		}
		time.Sleep(5 * time.Second)
	}
	uploadUrl, err := UploadAudioAliyun(saveName, savePath)
	if err != nil {
		err = errors.New("UploadAudioAliyun Err:" + err.Error())
		return
	}

	fileBody, err := ioutil.ReadFile(savePath)
	videoSize := len(fileBody)
	sizeFloat := (float64(videoSize) / float64(1024)) / float64(1024)
	sizeStr := utils.SubFloatToFloatStr(sizeFloat, 2)

	playSeconds, err := mp3duration.Calculate(savePath)
	if playSeconds <= 0 {
		playSeconds, err = utils.GetVideoPlaySeconds(savePath)
		if err != nil {
			err = errors.New("GetVideoPlaySeconds Err:" + err.Error())
			return
		}
	}

	if playSeconds > 0 {
		if utils.FileIsExist(savePath) {
			os.Remove(savePath)
		}
	}
	err = models.ModifyReportVideo(report.Id, uploadUrl, videoName, sizeStr, playSeconds)
	return
}

func GetChineseCount(str1 string) []string {
	fontArr := make([]string, 0)
	str := ""
	count := 0
	for _, char := range str1 {
		str += string(char)
		if unicode.Is(unicode.Han, char) {
			count++
			if count >= 1700 {
				fontArr = append(fontArr, str)
				str = ""
				count = 0
			}
		}
	}
	fontArr = append(fontArr, str)
	return fontArr
}

type BoxHeader struct {
	Size       uint32
	FourccType [4]byte
	Size64     uint64
}

func GetMP4Duration(reader io.ReaderAt) (lengthOfTime uint32, err error) {
	var info = make([]byte, 0x10)
	var boxHeader BoxHeader
	var offset int64 = 0
	for {
		_, err = reader.ReadAt(info, offset)
		if err != nil {
			return
		}
		boxHeader = getHeaderBoxInfo(info)
		fourccType := getFourccType(boxHeader)
		if fourccType == "moov" {
			break
		}
		if fourccType == "mdat" {
			if boxHeader.Size == 1 {
				offset += int64(boxHeader.Size64)
				continue
			}
		}
		offset += int64(boxHeader.Size)
	}
	moovStartBytes := make([]byte, 0x100)
	_, err = reader.ReadAt(moovStartBytes, offset)
	if err != nil {
		return
	}
	timeScaleOffset := 0x1C
	durationOffest := 0x20
	timeScale := binary.BigEndian.Uint32(moovStartBytes[timeScaleOffset : timeScaleOffset+4])
	Duration := binary.BigEndian.Uint32(moovStartBytes[durationOffest : durationOffest+4])
	lengthOfTime = Duration / timeScale
	return
}

func getHeaderBoxInfo(data []byte) (boxHeader BoxHeader) {
	buf := bytes.NewBuffer(data)
	binary.Read(buf, binary.BigEndian, &boxHeader)
	return
}

func getFourccType(boxHeader BoxHeader) (fourccType string) {
	fourccType = string(boxHeader.FourccType[:])
	return
}

func CreateReportVideo(reportTitle, reportContent, reportTime string) (uploadUrl, videoName, sizeStr string, playSeconds float64, err error) {
	defer func() {
		if err != nil {
			utils.FileLog.Error("CreateReportVideo Err:%s", err.Error())
			go alarm_msg.SendAlarmMsg("CreateReportVideo, reportTitle:"+reportTitle+", Err:"+err.Error(), 3)
		}
	}()
	if reportContent == "" {
		return
	}

	conf, e := models.GetBusinessConf()
	if e != nil {
		err = fmt.Errorf("获取基础配置失败, Err: " + e.Error())
		return
	}
	if conf[models.BusinessConfUseXf] != "true" {
		return
	}
	if conf[models.BusinessConfXfAppid] == "" || conf[models.BusinessConfXfApiKey] == "" || conf[models.BusinessConfXfApiSecret] == "" || conf[models.BusinessConfXfVcn] == "" {
		return
	}
	var xfReq XfParams
	xfReq.XfAPPID = conf[models.BusinessConfXfAppid]
	xfReq.XfAPIKey = conf[models.BusinessConfXfApiKey]
	xfReq.XfAPISecret = conf[models.BusinessConfXfApiSecret]

	ct, err := time.Parse(utils.FormatDateTime, reportTime)
	if err != nil {
		return
	}
	createTime := ct.Format("0102")
	videoName = reportTitle + "(" + createTime + ")"
	content := html.UnescapeString(reportContent)
	content = strings.Replace(content, "Powered", "", -1)
	content = strings.Replace(content, "by", "", -1)
	content = strings.Replace(content, "Froala", "", -1)
	content = strings.Replace(content, "Editor", "", -1)
	doc, err := goquery.NewDocumentFromReader(strings.NewReader(content))
	if err != nil {
		return
	}

	param := new(models.XfSendParam)
	param.Common.AppId = conf[models.BusinessConfXfAppid]
	param.Business.Aue = "lame"
	param.Business.Sfl = 1
	param.Business.Auf = "audio/L16;rate=16000"
	param.Business.Vcn = conf[models.BusinessConfXfVcn]
	param.Business.Speed = 50
	param.Business.Volume = 100
	param.Business.Pitch = 50
	param.Business.Bgs = 0
	param.Business.Tte = "UTF8"
	param.Business.Reg = "2"
	param.Business.Rdn = "0"
	param.Data.Status = 2
	videoContent := doc.Text()

	saveName := utils.GetRandStringNoSpecialChar(16) + ".mp3"
	savePath := "./" + saveName
	contentArr := GetChineseCount(videoContent)
	for _, v := range contentArr {
		newText := v
		param.Data.Text = base64.StdEncoding.EncodeToString([]byte(newText))
		result, tmpErr := json.Marshal(param)
		if tmpErr != nil {
			return
		}
		err = GetXfVideo(result, savePath, xfReq)
		if err != nil {
			err = errors.New("GetXfVideo Err:" + err.Error())
			utils.FileLog.Error("GetXfVideo err", err.Error())
			return
		}
		time.Sleep(5 * time.Second)
	}
	uploadUrl, err = UploadAudioAliyun(saveName, savePath)
	if err != nil {
		err = errors.New("UploadAudioAliyun Err:" + err.Error())
		return
	}

	fileBody, err := ioutil.ReadFile(savePath)
	videoSize := len(fileBody)
	sizeFloat := (float64(videoSize) / float64(1024)) / float64(1024)
	sizeStr = utils.SubFloatToFloatStr(sizeFloat, 2)

	playSeconds, err = mp3duration.Calculate(savePath)
	if playSeconds <= 0 {
		playSeconds, err = utils.GetVideoPlaySeconds(savePath)
		if err != nil {
			err = errors.New("GetVideoPlaySeconds Err:" + err.Error())
			return
		}
	}

	if playSeconds > 0 {
		if utils.FileIsExist(savePath) {
			os.Remove(savePath)
		}
	}

	return
}
